package emulator.vz200;

public class CassetteOut implements SignalEventSource, LineControlListener
{
  private final SignalEventQueue eventQueue;
  private final short[] elongation;
  private boolean muted;

  private CassetteOut()
  {
    throw new UnsupportedOperationException("unsupported empty constructor");
  }

  public CassetteOut(final long currentWallClockTime)
  {
    elongation = new short[4];
    eventQueue = new SignalEventQueue("cassette out", currentWallClockTime);
  }

  public void lineChanged(final SourceDataLineChangeEvent event)
  {
    eventQueue.reset(event.getCurrentWallClockTime());
  }

  public void resync()
  {
    eventQueue.resync();
  }

  public void volumeChanged(final double volume)
  {
    final int amplitude =
      (int)Math.min(Math.max(volume * 32767.0, 0.0), 32767.0);
    elongation[0] = 0;
    elongation[1] = (short)amplitude;
    elongation[2] = (short)amplitude;
    elongation[3] = (short)(2 * amplitude + 1);
    System.out.println(amplitude);
  }

  public void mutedChanged(final boolean muted)
  {
    this.muted = muted;
  }

  public long getAvailableNanoSeconds()
  {
    return eventQueue.getAvailableNanoSeconds();
  }

  public void putEvent(final int dataValue, final long wallClockTime)
  {
    if (!muted) {
      short signalValue = elongation[dataValue];
      eventQueue.put(signalValue, wallClockTime);
    }
  }

  public void getEvent(final SignalEventQueue.Event event,
                       final long maxTimeSpan)
  {
    if (maxTimeSpan <= 0) {
      throw new IllegalArgumentException("maxTimeSpan <= 0");
    }
    eventQueue.get(event, maxTimeSpan);
  }
}

/*
  Local Variables:
    coding:utf-8
    mode:Java
    End:
*/
